From ebde2a9d8050b8483da473af516a50222416c7ce Mon Sep 17 00:00:00 2001
From: Jingxian He <hejingxian@huawei.com>
Date: Tue, 29 Jun 2021 11:01:38 +0800
Subject: [PATCH 25/54] ignore special page dump

The special page dump will cost too long time when
thread num is very large. And special page dump
is not useful at every time.
Provide Ignore method for special page dump.

Signed-off-by: Jingxian He <hejingxian@huawei.com>
---
 criu/config.c             | 1 +
 criu/cr-dump.c            | 2 +-
 criu/crtools.c            | 1 +
 criu/include/cr_options.h | 1 +
 criu/seize.c              | 2 +-
 5 files changed, 5 insertions(+), 2 deletions(-)

diff --git a/criu/config.c b/criu/config.c
index 006753a..90d3951 100644
--- a/criu/config.c
+++ b/criu/config.c
@@ -549,6 +549,7 @@ int parse_options(int argc, char **argv, bool *usage_error,
 		BOOL_OPT("dump-char-dev", &opts.dump_char_dev),
 		BOOL_OPT("mask-exit-notify", &opts.mask_exit_notify),
 		BOOL_OPT("weak-file-check", &opts.weak_file_check),
+		BOOL_OPT("ignore-special-dump", &opts.ignore_special_dump),
 		{ },
 	};
 
diff --git a/criu/cr-dump.c b/criu/cr-dump.c
index e76fe5a..9627190 100644
--- a/criu/cr-dump.c
+++ b/criu/cr-dump.c
@@ -1780,7 +1780,7 @@ static int cr_dump_finish(int ret)
 	close_service_fd(CR_PROC_FD_OFF);
 	close_image_dir();
 
-	if (ret == 0 && opts.pin_memory) {
+	if (ret == 0 && opts.pin_memory && !opts.ignore_special_dump) {
 		pr_info("start restore_task_special_pages\n");
 		restore_task_special_pages(0);
 	}
diff --git a/criu/crtools.c b/criu/crtools.c
index 239464a..0a957b0 100644
--- a/criu/crtools.c
+++ b/criu/crtools.c
@@ -453,6 +453,7 @@ usage:
 "  --dump-char-dev     Dump char dev files as normal file with repair cmd\n"
 "  --mask-exit-notify     Mask task exit notify during dump and restore\n"
 "  --weak-file-check	Allow file size and mod larger than dumping value\n"
+"  --ignore-special-dump Ignore special task tid page dump\n"
 "\n"
 "Check options:\n"
 "  Without options, \"criu check\" checks availability of absolutely required\n"
diff --git a/criu/include/cr_options.h b/criu/include/cr_options.h
index fc7818c..681b519 100644
--- a/criu/include/cr_options.h
+++ b/criu/include/cr_options.h
@@ -183,6 +183,7 @@ struct cr_options {
 	int			dump_char_dev;
 	int			mask_exit_notify;
 	int			weak_file_check;
+	int			ignore_special_dump;
 };
 
 extern struct cr_options opts;
diff --git a/criu/seize.c b/criu/seize.c
index 387681c..c615971 100644
--- a/criu/seize.c
+++ b/criu/seize.c
@@ -682,7 +682,7 @@ static void unseize_task_and_threads(const struct pstree_item *item, int st)
 	if (item->pid->state == TASK_DEAD)
 		return;
 
-	if (opts.pin_memory) {
+	if (opts.pin_memory && !opts.ignore_special_dump) {
         	for (i = 0; i < item->nr_threads; i++)
                 	dump_task_special_pages(item->threads[i].real);
 	}
-- 
2.33.1

